
import java.io.*;
import java.util.*;
import java.util.StringTokenizer;



class UConf {
	
	// The constants defined according to the URLD Protocol
	// DO NOT CHANGE THESE VALUES!!!
	// Mode values
	public static final short URL_MODE_DISABLE		= 0;
	public static final short URL_MODE_WRITE		= 1;
	public static final short URL_MODE_READ			= 2;
	public static final short URL_MODE_INTERACT		= 3;
	// Tags
	public static final short URL_TAG_NODE_NAME		= 91;
	public static final short URL_TAG_URL			= 92;
	public static final short URL_TAG_DESCRIPTION	= 93;
	public static final short URL_TAG_NODE_TYPE		= 94;
	public static final short URL_TAG_END			= 99;
	// Node type
	public static final short NODE_TYPE_UNK			= 0;
	public static final short NODE_TYPE_MNODE		= 1;
	public static final short NODE_TYPE_HA			= 2;
	// End of protocol-specific constants
	
	String confFile = "urld.conf";
	boolean success = false;
	
	// Default values used by URL Daemon
	int Port = 999;
	// We don't start doing anything till we are told to.
	int Mode = URL_MODE_DISABLE;
	// This implementation does not deal with multiple interfaces (yet)
	
	public UNode us;
	
	String HTMLFile;
	
	int SendTime = 10;
	int ExpireTime = 30;
	
	UDebug debug;
	
	public UConf(UDebug d)
	{
		debug = d;
		us = new UNode(d);
	}
	
	public boolean parseFile(String f) {
		String record = null;

		try {

		FileReader fr = new FileReader(f);
		BufferedReader br = new BufferedReader(fr);
		record = new String();

		while ((record = br.readLine()) != null) {
			StringTokenizer stTok = new StringTokenizer(record); 
				
			if (stTok.hasMoreTokens()) {
				String tok = stTok.nextToken();
				if (tok.equals("#")) continue;
				else if (tok.equals("sendIP")) {
					int SenderType = Integer.valueOf(stTok.nextToken().substring(0,1)).intValue();
					tok = stTok.nextToken();
					String SenderName = record.substring(record.indexOf(tok));
					debug.print("SendIP: SenderType is " + SenderType + " and its name is " + SenderName);
					us.setType(SenderType);
					us.setName(SenderName);
				}
				else if (tok.equals("mode")) {
					Mode = Integer.valueOf(stTok.nextToken()).intValue();
					if (Mode < URL_MODE_DISABLE || Mode > URL_MODE_INTERACT)
						Mode = URL_MODE_DISABLE;
					debug.print("Mode: " + Mode);
				}
				else if (tok.equals("port")) {
					Port = Integer.valueOf(stTok.nextToken()).intValue();
					debug.print("Port: " + Port);
				}
				else if (tok.equals("send")) {
					String url = stTok.nextToken();
					tok = stTok.nextToken();
					String description = record.substring(record.indexOf(tok));
					debug.print("URL: " + url + " ('" + description + "')");
					
					us.insert(url, description);
				}
				else if (tok.equals("sendTime")) {
					SendTime = Integer.valueOf(stTok.nextToken()).intValue();
					debug.print("sendTime: " + SendTime);
				}
				else if (tok.equals("recv")) {
					tok = stTok.nextToken();
					HTMLFile = record.substring(record.indexOf(tok));
					
					debug.print("recv: '" + HTMLFile + "'");
				}
				else if (tok.equals("expire")) {
					ExpireTime = Integer.valueOf(stTok.nextToken()).intValue();
					debug.print("expire: " + ExpireTime);
				}
				else if (tok.equals("interface")) {
					String url = stTok.nextToken();
					tok = stTok.nextToken();
					String description = record.substring(record.indexOf(tok));
					debug.print("interface: " + url + " ('" + description + "')");
				}
				else
				{
					String params = record.substring(record.indexOf(tok) + tok.length() + 1);
					debug.print("Unknown token '" + tok + "' has parameters : " + params);
				}
			}

		} // END WHILE

		} catch (IOException e) {

		// catch possible io errors from readLine()
		debug.print("IOException error: " + e.toString());
		} 

		return false;
	}
	
	public static void main(String args[]) {
		try {
			UConf t = new UConf(new UDebug(System.out));
			t.parseFile("e:\\psu-work\\urld\\urld.conf");
		}
		catch (Exception e) { System.out.println("Nope, didn't work.." + e.toString());}
	}
}
