import java.awt.*;
import java.awt.event.*;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JMenuBar;
import javax.swing.KeyStroke;

import javax.swing.JTextArea;
import javax.swing.JScrollPane;
import javax.swing.JFrame;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import java.io.File;
/*
 * Main Dialog box; starts the other stuff.
 */
public class URLD extends JFrame 
                      implements ActionListener {
    JTextArea output;
    JScrollPane scrollPane;
    String newline = "\n";
	
	UDebug debug;
	UConf conf;
	
	// The reader
	UReader reader;
	// The writer
	UWriter writer;
	
	private class MenuStuff {
			String name;
			int keyEvent;
			boolean initialState;
			JMenuItem jm;
			MenuStuff(String n, int k, boolean i)
			{
				name = n;
				keyEvent = k;
				initialState = i;
			}
	};
	
	final MenuStuff urldMenu[] = {
		new MenuStuff( "Select Configuration File..", KeyEvent.VK_C, true ),
		new MenuStuff(  "Start URL Daemon", KeyEvent.VK_S, false ),
		new MenuStuff(  "Stop URL Daemon", KeyEvent.VK_T, false ),
		new MenuStuff(  "Exit", KeyEvent.VK_E, true )
	};
	
	private class confChooser extends FileFilter {
		public boolean accept(File f) {
			if (f.isDirectory()) {
				return true;
			}

			String ext = null;
			String s = f.getName();
			int i = s.lastIndexOf('.');

			if (i > 0 &&  i < s.length() - 1) {
			    ext = s.substring(i+1).toLowerCase();
			}
			if (ext == null) return false;
			if (ext.equals("conf")) return true;
			return false;
		}
		
		public String getDescription()
		{
			return "Selects only conf files";
		}
	}
	
	final JFileChooser fc = new JFileChooser();
	
    public URLD() {
        JMenuBar menuBar;
        JMenu menu, submenu;
        JMenuItem menuItem;

		fc.setFileFilter(new confChooser());
		
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
				doCleanUp();
                System.exit(0);
            }
        });

        //Add regular components to the window, using the default BorderLayout.
        Container contentPane = getContentPane();
        output = new JTextArea(5, 30);
        output.setEditable(false);
        scrollPane = new JScrollPane(output);
        contentPane.add(scrollPane, BorderLayout.CENTER);
		
		debug = new UDebug(output);
	
		debug.print("Please select the configuration file (from the File menu) to begin." + newline);

        //Create the menu bar.
        menuBar = new JMenuBar();
        setJMenuBar(menuBar);

        //Build the first menu.
        menu = new JMenu("File");
        menu.setMnemonic(KeyEvent.VK_F);
        menu.getAccessibleContext().setAccessibleDescription(
                "Control the various options of URL Daemon");
        menuBar.add(menu);

		for (int i = 0; i < urldMenu.length; i++) {
			menuItem = new JMenuItem(urldMenu[i].name);
			urldMenu[i].jm = menuItem;
			menuItem.setMnemonic(urldMenu[i].keyEvent);
			menuItem.addActionListener(this);
			menuItem.setEnabled(urldMenu[i].initialState);
			menu.add(menuItem);
		}
		
        //Build second menu in the menu bar.
        menu = new JMenu("About");
        menu.setMnemonic(KeyEvent.VK_A);
        menu.getAccessibleContext().setAccessibleDescription(
                "About the URL Daemon");

		menuItem = new JMenuItem("About the URL Daemon..");
        menuItem.setMnemonic(KeyEvent.VK_E);
        menuItem.addActionListener(this);
        menu.add(menuItem);

		menuBar.add(menu);
    }

    public void actionPerformed(ActionEvent e) {
        JMenuItem source = (JMenuItem)(e.getSource());
        /*String s = "Action event detected."
                   + newline
                   + "    Event source: " + source.getText()
                   + " (an instance of " + getClassName(source) + ")";
        output.append(s + newline);*/
		int i;
		for (i = 0; i < urldMenu.length; i++)
			if (urldMenu[i].name.equals(source.getText())) break;
		
		switch(i) {
		case 0:
			// Do the configuration file handling here
			int returnVal = fc.showOpenDialog(this);

			if (returnVal == JFileChooser.APPROVE_OPTION) {
			    File file = fc.getSelectedFile();
			    //this is where a real application would open the file.
			    debug.print("Opening: " + file.getName() + "." + newline);
				conf = new UConf(debug);
				debug.shh();
				conf.parseFile(file.getAbsolutePath());
				debug.verbose();
				debug.print("The selected mode is " + conf.Mode);
				if ((conf.Mode & 1) != 0)
				{
					debug.print("For writing, we use the following parameters:");
					debug.print("Broadcast is " + conf.us.doBroadcast);
					debug.print("Multicast is " + conf.us.doMulticast);
					debug.print("The URLs loaded are: ");
					for (i = 0; i < conf.us.getURLCount(); i++)
					{
						debug.print(conf.us.getURL(i) + " ( " + conf.us.getDesc(i) + " ) ");
					}
					debug.print("We will write to port " + conf.Port);
					debug.print("This will be sent every " + conf.SendTime + " seconds.");
					if (writer != null)
					{
						writer.stopCompletely();
						try {
							writer.join();
						} catch (InterruptedException ie) {
							writer = null;
						}
					}
					writer = new UWriter(conf, debug);
					writer.pauseThread();
					writer.start();
				}
				if ((conf.Mode & 2) != 0)
				{
					debug.print("For reading, we use the following parameters:");
					debug.print("The generated file is " + conf.HTMLFile);
					debug.print("We will listen to port " + conf.Port);
					debug.print("The nodes expire every " + conf.ExpireTime + " seconds.");
					if (reader != null) {
						reader.stopCompletely();
						reader = null;
					}
					reader = new UReader(conf, debug);
					reader.start();
				}
				// Enable the Start menu and disable the Configuration menu
				urldMenu[0].jm.setEnabled(false);
				urldMenu[1].jm.setEnabled(true);
				urldMenu[2].jm.setEnabled(false);
				debug.print("Select 'Start' to start the URL Daemon.");
			} else {
			    debug.print("Open command cancelled by user." + newline);
			}
			break;
		case 1:
			debug.print("Select 'Stop' to stop the URL Daemon.");
			// Start the URL Daemon
			urldMenu[1].jm.setEnabled(false);
			urldMenu[2].jm.setEnabled(true);
			if ((conf.Mode & 1) != 0)
				writer.resumeThread();
			if ((conf.Mode & 2) != 0)
				reader.resumeThread();
			break;
		case 2:
			debug.print("URL Daemon being stopped.");
			// Stop the URL Daemon
			urldMenu[0].jm.setEnabled(true);
			urldMenu[1].jm.setEnabled(true);
			urldMenu[2].jm.setEnabled(false);
			if ((conf.Mode & 1) != 0)
				writer.pauseThread();
			if ((conf.Mode & 2) != 0)
				reader.pauseThread();
			break;
		case 3:
			doCleanUp();
			System.exit(0);
			break;
		default:
			// About box
			break;
		}
    }
	
	private void doCleanUp()
	{
		debug.print("Cleaning up..");
		try {
			Thread.sleep(500);
		} catch (InterruptedException e) {}
	}

    public static void main(String[] args) {
        URLD window = new URLD();

        window.setTitle("URL Daemon");
        window.setSize(450, 260);
        window.setVisible(true);
    }
}
